@@ -27,6 +27,7 @@
 using System;
 using System.Collections;
 using System.IO;
+using System.Diagnostics;
 using System.Reflection;
 using System.Xml.Serialization;
 using System.Text.RegularExpressions;
@@ -334,60 +335,32 @@ namespace Beagle.Util {
 				}
 			}
 
-			// Best window position and dimension
+			// BeagleSearch window position and dimension
 			// stored as percentage of screen co-ordinates
 			// to deal with change of resolution problem - hints from tberman
 
-			private float best_pos_x = 0;
-			public float BestPosX {
-				get { return best_pos_x; }
-				set { best_pos_x = value; }
+			private float beagle_search_pos_x = 0;
+			public float BeaglePosX {
+				get { return beagle_search_pos_x; }
+				set { beagle_search_pos_x = value; }
 			}
 			
-			private float best_pos_y = 0;
-			public float BestPosY {
-				get { return best_pos_y; }
-				set { best_pos_y = value; }
+			private float beagle_search_pos_y = 0;
+			public float BeaglePosY {
+				get { return beagle_search_pos_y; }
+				set { beagle_search_pos_y = value; }
 			}
 			
-			// dont explicitly set height on first run
-			private float best_width = 0; 
-			public float BestWidth {
-				get { return best_width; }
-				set { best_width = value; }
+			private float beagle_search_width = 0; 
+			public float BeagleSearchWidth {
+				get { return beagle_search_width; }
+				set { beagle_search_width = value; }
 			}
 
-			private float best_height = 0; // -ditto-
-			public float BestHeight {
-				get { return best_height; }
-				set { best_height = value; }
-			}
-
-			// FIXME Change names when Holmes is released under new name
-			// Holmes window position and dimension
-
-			private float holmes_pos_x = 0;
-			public float HolmesPosX {
-				get { return holmes_pos_x; }
-				set { holmes_pos_x = value; }
-			}
-			
-			private float holmes_pos_y = 0;
-			public float HolmesPosY {
-				get { return holmes_pos_y; }
-				set { holmes_pos_y = value; }
-			}
-			
-			private float holmes_width = 0; 
-			public float HolmesWidth {
-				get { return holmes_width; }
-				set { holmes_width = value; }
-			}
-
-			private float holmes_height = 0;
-			public float HolmesHeight {
-				get { return holmes_height; }
-				set { holmes_height = value; }
+			private float beagle_search_height = 0;
+			public float BeagleSearchHeight {
+				get { return beagle_search_height; }
+				set { beagle_search_height = value; }
 			}
 
 			// ah!We want a Queue but Queue doesnt serialize *easily*
@@ -407,18 +380,21 @@ namespace Beagle.Util {
 				set { static_queryables = value; }
 			}
 
-			private ArrayList allowed_backends = new ArrayList ();
-			public ArrayList AllowedBackends {
-				get { return allowed_backends; }
-				set { allowed_backends = value; }
-			}
-
+			// By default, every backend is allowed.
+			// Only maintain a list of denied backends.
 			private ArrayList denied_backends = new ArrayList ();
 			public ArrayList DeniedBackends {
 				get { return denied_backends; }
 				set { denied_backends = value; }
 			}
 
+			private bool allow_static_backend = false; // by default, false
+			public bool AllowStaticBackend {
+				get { return allow_static_backend; }
+				// Don't really want to expose this, but serialization requires it
+				set { allow_static_backend = value; }
+			}
+
 			private bool index_synchronization = true;
 			public bool IndexSynchronization {
 				get { return index_synchronization; }
@@ -426,6 +402,22 @@ namespace Beagle.Util {
 				set { index_synchronization = value; }
 			}
 
+			[ConfigOption (Description="Enable a backend", Params=1, ParamsDescription="Name of the backend to enable")]
+			internal bool AllowBackend (out string output, string [] args)
+			{
+				denied_backends.Remove (args [0]);
+				output = "Backend allowed (need to restart beagled for changes to take effect).";
+				return true;
+			}
+
+			[ConfigOption (Description="Disable a backend", Params=1, ParamsDescription="Name of the backend to disable")]
+			internal bool DenyBackend (out string output, string [] args)
+			{
+				denied_backends.Add (args [0]);
+				output = "Backend disabled (need to restart beagled for changes to take effect).";
+				return true;
+			}
+			
 			private bool allow_root = false;
 			public bool AllowRoot {
 				get { return allow_root; }
@@ -457,6 +449,14 @@ namespace Beagle.Util {
 				return true;
 			}
 
+			[ConfigOption (Description="Toggles whether static indexes will be enabled")]
+			internal bool ToggleAllowStaticBackend (out string output, string [] args)
+			{
+				allow_static_backend = !allow_static_backend;
+				output = "Static indexes are " + ((allow_static_backend) ? "enabled" : "disabled") + " (need to restart beagled for changes to take effect).";
+				return true;
+			}		
+
 			[ConfigOption (Description="Toggles whether your indexes will be synchronized locally if your home directory is on a network device (eg. NFS/Samba)")]
 			internal bool ToggleIndexSynchronization (out string output, string [] args)
 			{
@@ -659,6 +659,7 @@ namespace Beagle.Util {
 				output = "PublicFolder " + args[0] + " removed.";
 				return true;
 			}			
+
 		}
 
 		[ConfigSection (Name="networking")]
