@@ -316,37 +316,37 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			indexable.MimeType = "message/rfc822";
 			indexable.CacheContent = false;
 
-			indexable.AddProperty (Property.NewKeyword ("fixme:client", "evolution"));
-			indexable.AddProperty (Property.NewKeyword ("fixme:account", "Local"));
-                        indexable.AddProperty (Property.NewKeyword ("fixme:folder", this.folder_name));
+			indexable.AddProperty (Property.NewUnsearched ("fixme:client", "evolution"));
+			indexable.AddProperty (Property.NewUnsearched ("fixme:account", "Local"));
+                        indexable.AddProperty (Property.NewUnsearched ("fixme:folder", this.folder_name));
 
 			GMime.InternetAddressList addrs;
 
 			addrs = message.GetRecipients (GMime.Message.RecipientType.To);
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (this.folder_name == "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
-					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = message.GetRecipients (GMime.Message.RecipientType.Cc);
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (this.folder_name == "Sent"  && ia.AddressType != GMime.InternetAddressType.Group)
-					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = GMime.InternetAddressList.ParseString (GMime.Utils.HeaderDecodePhrase (message.Sender));
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (this.folder_name != "Sent"  && ia.AddressType != GMime.InternetAddressType.Group)
-					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:gotFrom", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
 
-	                Property flag_prop = Property.NewKeyword ("fixme:flags", flags);
+	                Property flag_prop = Property.NewUnsearched ("fixme:flags", flags);
 			flag_prop.IsMutable = true;
 			indexable.AddProperty (flag_prop);
 
@@ -421,6 +421,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		private string folder_cache_name;
 		private Hashtable mapping;
 		private ArrayList deleted_list;
+		private bool delete_mode;
+		private int delete_count;
 
 		public EvolutionMailIndexableGeneratorImap (EvolutionMailQueryable queryable, FileInfo summary_info) : base (queryable)
 		{
@@ -650,26 +652,20 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			if (this.summary_enumerator.MoveNext ())
 				return true;
 
-			Logger.Log.Debug ("Queuing up {0} removals for deleted messages in ", this.deleted_list.Count, this.folder_name);
-			foreach (string uid in this.deleted_list) {
-				Uri uri = EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, uid);
+			this.delete_mode = true;
 
-				// FIXME: This is kind of a hack, but it's the only way
-				// with the IndexableGenerator to handle our removals.
-				Scheduler.Task task = this.queryable.NewRemoveTask (uri);
-				task.Priority = Scheduler.Priority.Immediate;
-				this.queryable.ThisScheduler.Add (task);
-			}
+			if (this.deleted_list.Count > 0)
+				return true;
 
 			string progress = "";
 			if (this.count > 0 && this.summary.header.count > 0) {
-				progress = String.Format (" ({0}/{1} {2:###.0}%)",
+				progress = String.Format ("({0}/{1} {2:###.0}%)",
 							  this.count,
 							  this.summary.header.count,
 							  100.0 * this.count / this.summary.header.count);
 			}
 
-			Logger.Log.Debug ("{0}: Finished indexing {1} messages{2}", this.folder_name, this.indexed_count, progress);
+			Logger.Log.Debug ("{0}: Finished indexing {1} messages {2}, {3} messages deleted", this.folder_name, this.indexed_count, progress, this.delete_count);
 
 			this.SaveCache ();
 			this.CrawlFinished ();
@@ -688,6 +684,21 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		{
 			Indexable indexable = null;
 
+			// No more new messages to index, so start on the removals.
+			if (this.delete_mode) {
+				string uid = (string) this.deleted_list [0];
+				Uri uri = EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, uid);
+				
+				indexable = new Indexable (IndexableType.Remove, uri);
+
+				this.deleted_list.RemoveAt (0);
+				this.mapping.Remove (uid);
+
+				this.delete_count++;
+
+				return indexable;
+			}
+
 			Camel.MessageInfo mi = (Camel.MessageInfo) this.summary_enumerator.Current;
 
 			++this.count;
@@ -729,7 +740,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				indexable = new Indexable (uri);
 				indexable.Type = IndexableType.PropertyChange;
 
-				Property flag_prop = Property.NewKeyword ("fixme:flags", mi.flags);
+				Property flag_prop = Property.NewUnsearched ("fixme:flags", mi.flags);
 				flag_prop.IsMutable = true;
 				indexable.AddProperty (flag_prop);
 
@@ -771,9 +782,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			indexable.MimeType = "message/rfc822";
 			indexable.HitType = "MailMessage";
 
-			indexable.AddProperty (Property.NewKeyword ("fixme:account",  this.imap_name));
-                        indexable.AddProperty (Property.NewKeyword ("fixme:folder",   this.folder_name));
-			indexable.AddProperty (Property.NewKeyword ("fixme:client", "evolution"));
+			indexable.AddProperty (Property.NewUnsearched ("fixme:account",  this.imap_name));
+                        indexable.AddProperty (Property.NewUnsearched ("fixme:folder",   this.folder_name));
+			indexable.AddProperty (Property.NewUnsearched ("fixme:client", "evolution"));
 			
 			if (!have_content) {
 				indexable.AddProperty (Property.New ("dc:title", GMime.Utils.HeaderDecodePhrase (messageInfo.subject)));
@@ -784,7 +795,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			addrs = GMime.InternetAddressList.ParseString (messageInfo.to);
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
-					indexable.AddProperty (Property.NewKeyword ("fixme:to", ia.ToString (false)));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:to", ia.ToString (false)));
 					if (ia.AddressType != GMime.InternetAddressType.Group) {
 						indexable.AddProperty (Property.New ("fixme:to_address", ia.Addr));
 						indexable.AddProperty (Property.New ("fixme:to_sanitized", StringFu.SanitizeEmail (ia.Addr)));
@@ -793,14 +804,14 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				}
 
 				if (this.folder_name == "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
-					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = GMime.InternetAddressList.ParseString (messageInfo.cc);
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
-					indexable.AddProperty (Property.NewKeyword ("fixme:cc", ia.ToString (false)));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:cc", ia.ToString (false)));
 					if (ia.AddressType != GMime.InternetAddressType.Group) {
 						indexable.AddProperty (Property.New ("fixme:cc_address", ia.Addr));
 						indexable.AddProperty (Property.New ("fixme:cc_sanitized", StringFu.SanitizeEmail (ia.Addr)));
@@ -809,14 +820,14 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				}
 
 				if (this.folder_name == "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
-					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = GMime.InternetAddressList.ParseString (messageInfo.from);
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
-					indexable.AddProperty (Property.NewKeyword ("fixme:from", ia.ToString (false)));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:from", ia.ToString (false)));
 					if (ia.AddressType != GMime.InternetAddressType.Group) {
 						indexable.AddProperty (Property.New ("fixme:from_address", ia.Addr));
 						indexable.AddProperty (Property.New ("fixme:from_sanitized", StringFu.SanitizeEmail (ia.Addr)));
@@ -825,13 +836,13 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				}
 
 				if (this.folder_name != "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
-					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:gotFrom", ia.Addr));
 			}
 			addrs.Dispose ();
 
                         indexable.AddProperty (Property.NewKeyword ("fixme:mlist", messageInfo.mlist));
 
-			Property flag_prop = Property.NewKeyword ("fixme:flags", messageInfo.flags);
+			Property flag_prop = Property.NewUnsearched ("fixme:flags", messageInfo.flags);
 			flag_prop.IsMutable = true;
 			indexable.AddProperty (flag_prop);
 
