@@ -37,7 +37,8 @@ namespace Beagle.Daemon {
 		private string   path;
 		private DateTime last_mtime;
 
-		private DateTime last_indexed;
+		private DateTime last_attrtime;
+
 		private string   filter_name;
 		private int      filter_version;
 		
@@ -47,7 +48,7 @@ namespace Beagle.Daemon {
 		}
 
 		public string Directory {
-			get { return System.IO.Path.GetDirectoryName (path); }
+			get { return FileSystem.GetDirectoryNameRootOk (path); }
 			set { path = System.IO.Path.Combine (value, Filename); }
 		}
 
@@ -58,7 +59,11 @@ namespace Beagle.Daemon {
 
 		public string Path {
 			get { return path; }
-			set { path = value != null ? System.IO.Path.GetFullPath (value) : null; }
+			set { 
+				path = value != null ? System.IO.Path.GetFullPath (value) : null;
+				if (path != null && path != "/" && path.EndsWith ("/"))
+					path = path.TrimEnd ('/');
+			}
 		}
 
 		public DateTime LastWriteTime {
@@ -66,9 +71,10 @@ namespace Beagle.Daemon {
 			set { last_mtime = value; }
 		}
 
-		public DateTime LastIndexedTime {
-			get { return last_indexed; }
-			set { last_indexed = value; }
+		// When the attributes were last written out.
+		public DateTime LastAttrTime {
+			get { return last_attrtime; }
+			set { last_attrtime = value; }
 		}
 
 		public bool HasFilterInfo {
