@@ -37,6 +37,8 @@ using Beagle.Util;
 namespace Beagle.Daemon.FileSystemQueryable {
 
 	[QueryableFlavor (Name="Files", Domain=QueryDomain.Local, RequireInotify=false)]
+	[PropertyKeywordMapping (Keyword="extension", PropertyName="beagle:FilenameExtension", IsKeyword=true, Description="File extension, e.g. extension:jpeg. Use extension: to search in files with no extension.")]
+	[PropertyKeywordMapping (Keyword="ext", PropertyName="beagle:FilenameExtension", IsKeyword=true, Description="File extension, e.g. ext:jpeg. Use ext: to search in files with no extension.")]
 	public class FileSystemQueryable : LuceneQueryable {
 
 		static public bool Debug = false;
@@ -434,6 +436,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			if (filter.Ignore (parent, name, true))
 				return;
 
+			// FIXME: ! parent.HasChildWithName (name)
 			if (parent != null && parent.HasChildWithName (name))
 				return;
 
@@ -1029,6 +1032,9 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 			if (! File.Exists (path))
 				return;
+
+			if (FileSystem.IsSpecialFile (path))
+				return;
 			
 			if (filter.Ignore (dir, name, false))
 				return;
@@ -1376,20 +1382,18 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		{
 			// Uri remapping from a hit is easy: the internal uri
 			// is stored in a property.
-			Uri uri;
-			uri = UriFu.UriStringToUri (hit ["beagle:InternalUri"]);
+			Uri uri = UriFu.UriStringToUri (hit ["beagle:InternalUri"]);
 
-			string path;
-			path = TextCache.UserCache.LookupPathRaw (uri);
+			string path = TextCache.UserCache.LookupPathRaw (uri);
 
 			if (path == null)
 				return null;
 
 			// If this is self-cached, use the remapped Uri
 			if (path == TextCache.SELF_CACHE_TAG)
-				path = hit.Uri.LocalPath;
+				return SnippetFu.GetSnippetFromFile (query_terms, hit.Uri.LocalPath);
 
-			return SnippetFu.GetSnippetFromFile (query_terms, path);
+			return SnippetFu.GetSnippetFromTextCache (query_terms, path);
 		}
 
 		override public void Start ()
