@@ -42,20 +42,12 @@ namespace Beagle.Daemon {
 
 		static FilterFactory ()
 		{
-			string path = Environment.GetEnvironmentVariable ("BEAGLE_FILTER_PATH");
-			
-			if (path == null || path == "")
-				path = PathFinder.FilterDir;
-			else if (path [path.Length-1] == ':')
-				path += PathFinder.FilterDir;
-
-			Hashtable seen = new Hashtable ();
-
-			foreach (string dir in path.Split (':')) {
-				if (! seen.Contains (dir))
-					ScanDirectoryForAssemblies (dir);
-				seen [dir] = true;
-			}
+			ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_FILTER_PATH", PathFinder.FilterDir,
+								   delegate (Assembly a) {
+									   int n = ScanAssemblyForFilters (a);
+									   Logger.Log.Debug ("Loaded {0} filter{1} from {2}",
+											     n, n == 1 ? "" : "s", a.Location);
+								   });
 		}
 
 		/////////////////////////////////////////////////////////////////////////
@@ -118,7 +110,7 @@ namespace Beagle.Daemon {
 
 		static public ICollection CreateFiltersFromPath (string path)
 		{
-			string guessed_mime_type = GnomeFu.GetMimeType (path);
+			string guessed_mime_type = XdgMime.GetMimeType (path);
 			string extension = Path.GetExtension (path);
 			return CreateFilters (UriFu.PathToFileUri (path), extension, guessed_mime_type);
 		}
@@ -149,7 +141,17 @@ namespace Beagle.Daemon {
 		static public TextReader FilterFile (string path, string mime_type)
 		{
 			if (mime_type == null)
-				mime_type = GnomeFu.GetMimeType (path);
+				mime_type = XdgMime.GetMimeType (path);
+			else {
+				// Mime types are all lower-case by convention
+				// and all the filters expect them that way,
+				// but we can't trust that they already are
+				// when we get it from somewhere other than
+				// xdgmime.  They are often capitalized in
+				// emails, for example.
+
+				mime_type = mime_type.ToLower ();
+			}
 
 			if (mime_type == null)
 				return null;
@@ -216,12 +218,12 @@ namespace Beagle.Daemon {
 			if (indexable.MimeType != null)
 				filters = CreateFiltersFromMimeType (indexable.MimeType);
 
-			if (indexable.IsNonTransient) {
+			if (indexable.ContentUri.IsFile) {
 				path = indexable.ContentUri.LocalPath;
 
 				// Otherwise sniff the mime-type from the file
 				if (indexable.MimeType == null)
-					indexable.MimeType = GnomeFu.GetMimeType (path);
+					indexable.MimeType = XdgMime.GetMimeType (path);
 
 				if (filters == null || filters.Count == 0) {
 					filters = CreateFiltersFromIndexable (indexable);
@@ -230,9 +232,12 @@ namespace Beagle.Daemon {
 				if (Directory.Exists (path)) {
 					indexable.MimeType = "inode/directory";
 					indexable.NoContent = true;
-					indexable.Timestamp = Directory.GetLastWriteTimeUtc (path);
+					if (! indexable.ValidTimestamp)
+						indexable.Timestamp = Directory.GetLastWriteTimeUtc (path);
 				} else if (File.Exists (path)) {
-					indexable.Timestamp = File.GetLastWriteTimeUtc (path);
+					// Set the timestamp to the best possible estimate (if no timestamp was set by the backend)
+					if (! indexable.ValidTimestamp)
+						indexable.Timestamp = File.GetLastWriteTimeUtc (path);
 				} else {
 					Logger.Log.Warn ("No such file: {0}", path);
 					return false;
@@ -244,7 +249,7 @@ namespace Beagle.Daemon {
 				if (! indexable.NoContent) {
 					indexable.NoContent = true;
 
-					Logger.Log.Debug ("No filter for {0}", path);
+					Logger.Log.Debug ("No filter for {0} ({1})", path != null ? path : indexable.Uri.ToString (), indexable.MimeType);
 					return false;
 				}
 				
@@ -332,54 +337,32 @@ namespace Beagle.Daemon {
 		{
 			int count = 0;
 
-			foreach (Type t in assembly.GetTypes ()) {
-				if (t.IsSubclassOf (typeof (Filter)) && ! t.IsAbstract) {
-					Filter filter = null;
+			foreach (Type t in ReflectionFu.ScanAssemblyForClass (assembly, typeof (Filter))) {
+				Filter filter = null;
 
-					try {
-						filter = (Filter) Activator.CreateInstance (t);
-					} catch (Exception ex) {
-						Logger.Log.Error ("Caught exception while instantiating {0}", t);
-						Logger.Log.Error (ex);
-					}
-
-					if (filter == null)
-						continue;
-
-					filter_versions_by_name [t.ToString ()] = filter.Version;
-
-					foreach (FilterFlavor flavor in filter.SupportedFlavors) {
-						filter_types_by_flavor [flavor] = t;
-						FilterFlavor.Flavors.Add (flavor);
-					}
-
-					++count;
+				try {
+					filter = (Filter) Activator.CreateInstance (t);
+				} catch (Exception ex) {
+					Logger.Log.Error ("Caught exception while instantiating {0}", t);
+					Logger.Log.Error (ex);
 				}
+
+				if (filter == null)
+					continue;
+
+				filter_versions_by_name [t.ToString ()] = filter.Version;
+
+				foreach (FilterFlavor flavor in filter.SupportedFlavors) {
+					filter_types_by_flavor [flavor] = t;
+					FilterFlavor.Flavors.Add (flavor);
+				}
+
+				++count;
 			}
-			
+
 			return count;
 		}
-
-		static private void ScanDirectoryForAssemblies (string dir)
-		{
-			if (dir == null || dir == "")
-				return;
-
-			if (! Directory.Exists (dir)) {
-				Logger.Log.Debug ("'{0}' is not a directory: No filters loaded", dir);
-				return;
-			}
-			
-			DirectoryInfo dir_info = new DirectoryInfo (dir);
-			foreach (FileInfo file_info in dir_info.GetFiles ()) {
-				if (file_info.Extension == ".dll") {
-					Assembly a = Assembly.LoadFrom (file_info.FullName);
-					int n = ScanAssemblyForFilters (a);
-					Logger.Log.Debug ("Loaded {0} filter{1} from {2}",
-							  n, n == 1 ? "" : "s", file_info.FullName);
-				}
-			}
-		}
+		
 	}
 
 	/////////////////////////////////////////////////////////////////////////
