@@ -259,13 +259,15 @@ namespace Beagle.Daemon.GaimLogQueryable {
 		{
 			ImBuddy buddy = list.Search (hit ["fixme:speakingto"]);
 			
-			if (buddy != null) {
-				if (buddy.Alias != "")
-					hit ["fixme:speakingto_alias"] = buddy.Alias;
-				
-				if (buddy.BuddyIconLocation != "")
-					hit ["fixme:speakingto_icon"] = Path.Combine (icons_dir, buddy.BuddyIconLocation);
-			}
+			//We might still want to see a chat even if someones not on our buddy list
+			if (buddy == null) 
+				return true;
+			
+			if (buddy.Alias != "")
+ 				hit.AddProperty (Beagle.Property.NewKeyword ("fixme:speakingto_alias", buddy.Alias));
+ 				
+ 			if (buddy.BuddyIconLocation != "")
+ 				hit.AddProperty (Beagle.Property.NewUnsearched ("fixme:speakingto_icon", Path.Combine (icons_dir, buddy.BuddyIconLocation)));
 			
 			return true;
 		}
