@@ -152,6 +152,8 @@ namespace Beagle.Daemon {
 
 		private bool IsDanglingLock (FileInfo info)
 		{
+			Log.Debug ("Checking for dangling locks...");
+
 			// It isn't even a lock file
 			if (! info.Name.EndsWith (".lock"))
 				return false;
@@ -169,6 +171,16 @@ namespace Beagle.Daemon {
 				return false;
 			}
 
+			
+			if (pid == null) {
+				// Looks like the lock file was empty, which really
+				// shouldn't happen.  It should contain the PID of
+				// the process which locked it.  Lets be on the safe
+				// side and assume it's a dangling lock.
+				Log.Warn ("Found an empty lock file, that shouldn't happen: {0}", info.FullName);
+				return true;
+			}
+
 			string cmdline_file;
 			cmdline_file = String.Format ("/proc/{0}/cmdline", pid);
 			
@@ -206,13 +218,6 @@ namespace Beagle.Daemon {
 			return false;
 		}
 		
-
-		// Return true if there are dangling locks
-		protected bool HaveDanglingLocks ()
-		{
-			return false;
-		}
-
 		protected bool Exists ()
 		{
 			if (! (Directory.Exists (top_dir)
