@@ -31,6 +31,7 @@
 
 using System;
 using System.Collections;
+using System.IO;
 
 using Beagle.Util;
 
@@ -62,6 +63,9 @@ namespace Beagle.Daemon {
 			file_references_count [path] = (int) file_references_count [path] + 1;
 		}
 		
+		// returns
+		// true : reference left
+		// false: no more reference left
 		private bool DecrementReferenceCount (string path)
 		{
 			if (! file_references_count.Contains (path))
@@ -95,8 +99,10 @@ namespace Beagle.Daemon {
 			if (indexable.Type == IndexableType.Remove)
 				return true;
 
-			// Remember the file's info and mtime.
-			CachedFileInfo info = new CachedFileInfo ();
+			CachedFileInfo info = (CachedFileInfo) file_info_cache [indexable.Uri];
+
+			if (info == null)
+				info = new CachedFileInfo ();
 
 			info.Uri = indexable.Uri;
 
@@ -110,14 +116,20 @@ namespace Beagle.Daemon {
 				IncrementReferenceCount (info.Path);
 			}
 
+			// The path could be null in certain cases:
+			//    * The indexable is a non-file URI and no
+			//      parent URI is set.
+			//    * The indexable is a child indexable and the
+			//      parent URI is not a file URI.
+			if (info.Path == null)
+				return true;
+
 			try {
-				info.Mtime = FileSystem.GetLastWriteTime (info.Path);
-			} catch (Exception ex) {
+				info.Mtime = FileSystem.GetLastWriteTimeUtc (info.Path);
+			} catch (FileNotFoundException ex) {
 				// If we can't get an mtime for the file, it must
 				// have disappeared out from under us.  In that case,
 				// don't bother adding anything.
-				// FIXME: Should we be more specific and only catch
-				// a file not found error here?
 				return false;
 			}
 
@@ -136,7 +148,7 @@ namespace Beagle.Daemon {
 
 			// Yeah, this is ghetto. If it's a file that's shared across multiple
 			// indexables, only tag it with when the last indexable has been indexed.
-			if (info.Shared && ! DecrementReferenceCount (info.Path))
+			if (info.Shared && DecrementReferenceCount (info.Path))
 				return;
 
 			// Since we know that the file has been successfully
@@ -157,6 +169,8 @@ namespace Beagle.Daemon {
 
 			if (! FileAttributesStore.Write (attr))
 				Logger.Log.Error ("Couldn't write attributes for {0}", info.Path);
+
+			file_info_cache.Remove (info.Uri);
 		}
 
 		override protected void PostRemoveHook (Indexable indexable, IndexerRemovedReceipt receipt)
