@@ -35,41 +35,73 @@ namespace Beagle.Daemon {
 	
 	public class QueryDriver {
 
-		// Enable or Disable specific Queryables by name
+		// Contains list of queryables explicitly asked by --allow-backend or --backend name
+		// --allow-backend/--backend name : dont read config information and only start backend 'name'
+		static ArrayList excl_allowed_queryables = new ArrayList ();
 
-		static ArrayList allowed_queryables = new ArrayList ();
+		// Contains list of denied queryables from config/arguments (every queryable is enabled by default)
+		// Unless overruled by --allow-backend/--backend name, deny backend only if names appears here.
 		static ArrayList denied_queryables = new ArrayList ();
+		
+		static bool to_read_conf = true; // read backends from conf if true
+		static bool done_reading_conf = false;
 
+		static private void ReadBackendsFromConf ()
+		{
+			if (! to_read_conf || done_reading_conf)
+				return;
+
+			// set flag here to stop Allow() from calling ReadBackendsFromConf() again
+			done_reading_conf = true;
+
+			// To allow static indexes, "static" should be in allowed_queryables
+			if (Conf.Daemon.AllowStaticBackend)
+				Allow ("static");
+
+			if (Conf.Daemon.DeniedBackends == null)
+				return;
+			
+			foreach (string name in Conf.Daemon.DeniedBackends)
+				denied_queryables.Add (name.ToLower ());
+		}
+
+		static public void OnlyAllow (string name)
+		{
+			excl_allowed_queryables.Add (name.ToLower ());
+			to_read_conf = false;
+		}
+		
 		static public void Allow (string name)
 		{
-			allowed_queryables.Add (name.ToLower ());
+			if (! done_reading_conf && to_read_conf)
+				ReadBackendsFromConf ();
+
+			denied_queryables.Remove (name.ToLower ());
 		}
 		
 		static public void Deny (string name)
 		{
-			denied_queryables.Add (name.ToLower ());
+			if (! done_reading_conf && to_read_conf)
+				ReadBackendsFromConf ();
+
+			name = name.ToLower ();
+			if (!denied_queryables.Contains (name))
+				denied_queryables.Add (name);
 		}
 
 		static private bool UseQueryable (string name)
 		{
-			if (allowed_queryables.Count == 0
-			    && denied_queryables.Count == 0)
-				return true;
-
 			name = name.ToLower ();
 
-			if (allowed_queryables.Count > 0) {
-				foreach (string allowed in allowed_queryables)
-					if (name == allowed)
-						return true;
+			if (excl_allowed_queryables.Contains (name))
+				return true;
+			if (excl_allowed_queryables.Count != 0)
+				return false;
+
+			if (denied_queryables.Contains (name))
 				return false;
-			}
 
-			foreach (string denied in denied_queryables)
-				if (name == denied)
-					return false;
 			return true;
-
 		}
 
 		//////////////////////////////////////////////////////////////////////////////////////
@@ -122,8 +154,9 @@ namespace Beagle.Daemon {
 		{
 			int count = 0;
 
-			foreach (Type type in GetQueryableTypes (assembly)) {
-				foreach (QueryableFlavor flavor in GetQueryableFlavors (type)) {
+			foreach (Type type in ReflectionFu.ScanAssemblyForInterface (assembly, typeof (IQueryable))) {
+				bool type_accepted = false;
+				foreach (QueryableFlavor flavor in ReflectionFu.ScanTypeForAttribute (type, typeof (QueryableFlavor))) {
 					if (! UseQueryable (flavor.Name))
 						continue;
 
@@ -150,11 +183,63 @@ namespace Beagle.Daemon {
 						queryables.Add (q);
 						iqueryable_to_queryable [iq] = q;
 						++count;
+						type_accepted = true;
 						break;
 					}
 				}
+
+				if (! type_accepted)
+					continue;
+
+				object[] attributes = type.GetCustomAttributes (false);
+				foreach (object attribute in attributes) {
+					PropertyKeywordMapping mapping = attribute as PropertyKeywordMapping;
+					if (mapping == null)
+						continue;
+					//Logger.Log.Debug (mapping.Keyword + " => " 
+					//		+ mapping.PropertyName + 
+					//		+ " is-keyword=" + mapping.IsKeyword + " (" 
+					//		+ mapping.Description + ") "
+					//		+ "(" + type.FullName + ")");
+					PropertyKeywordFu.RegisterMapping (mapping);
+				}
+					
+			}
+			Logger.Log.Debug ("Found {0} backends in {1}", count, assembly.Location);
+		}
+
+		////////////////////////////////////////////////////////
+
+		public static void ReadKeywordMappings ()
+		{
+			Logger.Log.Debug ("Reading mapping from filters");
+			ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_FILTER_PATH", PathFinder.FilterDir);
+
+			foreach (Assembly assembly in assemblies) {
+				foreach (Type type in assembly.GetTypes ())
+					if (type.FullName.StartsWith ("Beagle.Filters")) {
+
+						if (type.IsNestedPrivate ||
+						    type.IsNestedPublic ||
+						    type.IsNestedAssembly ||
+						    type.IsNestedFamily)
+							continue;
+
+						object[] attributes = type.GetCustomAttributes (false);
+						foreach (object attribute in attributes) {
+
+							PropertyKeywordMapping mapping = attribute as PropertyKeywordMapping;
+							if (mapping == null)
+								continue;
+							//Logger.Log.Debug (mapping.Keyword + " => " 
+							//		+ mapping.PropertyName
+							//		+ " is-keyword=" + mapping.IsKeyword + " (" 
+							//		+ mapping.Description + ") "
+							//		+ "(" + type.FullName + ")");
+							PropertyKeywordFu.RegisterMapping (mapping);
+						}
+					}
 			}
-			Logger.Log.Debug ("Found {0} types in {1}", count, assembly.FullName);
 		}
 
 		////////////////////////////////////////////////////////
@@ -166,6 +251,8 @@ namespace Beagle.Daemon {
 			if (!Directory.Exists (PathFinder.SystemIndexesDir))
 				return;
 			
+			Logger.Log.Info ("Loading system static indexes.");
+
 			int count = 0;
 
 			foreach (DirectoryInfo index_dir in new DirectoryInfo (PathFinder.SystemIndexesDir).GetDirectories ()) {
@@ -176,7 +263,7 @@ namespace Beagle.Daemon {
 					count++;
 			}
 
-			Logger.Log.Debug ("Found {0} system-wide indexes", count);
+			Logger.Log.Info ("Found {0} system-wide indexes.", count);
 		}
 
 		// Scans configuration for user-specified index paths 
@@ -186,6 +273,7 @@ namespace Beagle.Daemon {
 			int count = 0;
 
 			if (UseQueryable ("static")) {
+				Logger.Log.Info ("Loading user-configured static indexes.");
 				foreach (string path in Conf.Daemon.StaticQueryables)
 					static_queryables.Add (path);
 			}
@@ -201,7 +289,7 @@ namespace Beagle.Daemon {
 					count++;
 			}
 
-			Logger.Log.Debug ("Found {0} user-configured static queryables", count);
+			Logger.Log.Info ("Found {0} user-configured static indexes..", count);
 		}
 
 		// Instantiates and loads a StaticQueryable from an index directory
@@ -241,57 +329,17 @@ namespace Beagle.Daemon {
 
 		////////////////////////////////////////////////////////
 
-		static private Type[] GetQueryableTypes (Assembly assembly)
-		{
-			Type[] assembly_types = assembly.GetTypes ();
-			ArrayList types = new ArrayList (assembly_types.Length);
-
-			foreach (Type type in assembly_types)
-				if (TypeImplementsInterface (type, typeof (IQueryable)))
-					types.Add (type);
-		
-			return (Type[]) types.ToArray (typeof (Type));
-		}
-		
-		static private QueryableFlavor[] GetQueryableFlavors (Type type)
-		{
-			object[] attributes = Attribute.GetCustomAttributes (type);
-			ArrayList flavors = new ArrayList (attributes.Length);
-			
-			foreach (object obj in attributes) {
-					QueryableFlavor flavor = obj as QueryableFlavor;
-					if (flavor != null)
-						flavors.Add (flavor);
-			}
-
-			return (QueryableFlavor[]) flavors.ToArray (typeof (QueryableFlavor));
-		}
-
-		static private Assembly[] GetAssemblies ()
-		{
- 			Assembly[] assemblies;
-			int i = 0;
-			DirectoryInfo backends = new DirectoryInfo (PathFinder.BackendDir);
-
-			if (backends.Exists) {
-				FileInfo[] assembly_files = backends.GetFiles ("*.dll");
-				assemblies = new Assembly [assembly_files.Length + 1];
-
-				foreach (FileInfo assembly in assembly_files)
-					assemblies[i++] = Assembly.LoadFile (assembly.ToString ());
-
-			} else {
-				assemblies = new Assembly [1];
-			}
-
-			assemblies[i] = Assembly.GetExecutingAssembly ();
-		
-			return assemblies;
-		}
-
 		static public void Start ()
 		{
-			foreach (Assembly assembly in GetAssemblies ()) {
+			ReadBackendsFromConf ();
+
+			ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
+
+			// Only add the executing assembly if we haven't already loaded it.
+			if (assemblies.IndexOf (Assembly.GetExecutingAssembly ()) == -1)
+				assemblies.Add (Assembly.GetExecutingAssembly ());
+
+			foreach (Assembly assembly in assemblies) {
 				ScanAssembly (assembly);
 
 				// This allows backends to define their
@@ -299,6 +347,8 @@ namespace Beagle.Daemon {
 				Server.ScanAssemblyForExecutors (assembly);
 			}
 
+			ReadKeywordMappings ();
+
 			LoadSystemIndexes ();
 			LoadStaticQueryables ();
 
@@ -314,19 +364,30 @@ namespace Beagle.Daemon {
 		{
 			Logger.Log.Debug ("Starting queryables");
 
-			foreach (Queryable q in queryables)
+			foreach (Queryable q in queryables) {
+				Logger.Log.Info ("Starting backend: '{0}'", q.Name);
 				q.Start ();
+			}
 
 			return false;
 		}
 
 		static public string ListBackends ()
 		{
+			ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
+
+			// Only add the executing assembly if we haven't already loaded it.
+			if (assemblies.IndexOf (Assembly.GetExecutingAssembly ()) == -1)
+				assemblies.Add (Assembly.GetExecutingAssembly ());
+
 			string ret = "User:\n";
-			foreach (Assembly assembly in GetAssemblies ())
-				foreach (Type type in GetQueryableTypes (assembly))
-					foreach (QueryableFlavor flavor in GetQueryableFlavors (type))
-				ret += String.Format (" - {0}\n", flavor.Name);
+
+			foreach (Assembly assembly in assemblies) {
+				foreach (Type type in ReflectionFu.ScanAssemblyForInterface (assembly, typeof (IQueryable))) {
+					foreach (QueryableFlavor flavor in ReflectionFu.ScanTypeForAttribute (type, typeof (QueryableFlavor)))
+						ret += String.Format (" - {0}\n", flavor.Name);
+				}
+			}
 			
 			if (!Directory.Exists (PathFinder.SystemIndexesDir)) 
 				return ret;
