@@ -145,12 +145,16 @@ namespace Beagle.Daemon {
 					//Logger.Log.Debug ("Done sending request");
 				} catch (ResponseMessageException ex) {
 					Logger.Log.Debug ("Caught ResponseMessageException: {0}", ex.Message);
-				} catch (System.Net.Sockets.SocketException ex) {
-					Logger.Log.Debug ("Caught SocketException -- we probably need to launch a helper: {0}", ex.Message);
-					need_helper = true;
-				} catch (IOException ex) {
-					Logger.Log.Debug ("Caught IOException --- we probably need to launch a helper: {0}", ex.Message);
-					need_helper = true;
+
+					if (ex.InnerException is System.Net.Sockets.SocketException) {
+						Logger.Log.Debug ("InnerException is SocketException -- we probably need to launch a helper");
+						need_helper = true;
+					} else if (ex.InnerException is IOException) {
+						Logger.Log.Debug ("InnerException is IOException -- we probably need to launch a helper");
+						need_helper = true;
+					} else {
+						Logger.Log.Debug ("Exception was unexpected.  Details: {0}", ex);
+					}
 				}
 
 				// If we caught an exception...
@@ -216,26 +220,32 @@ namespace Beagle.Daemon {
 				
 				Logger.Log.Debug ("Launching helper process");
 
-				Process p = new Process ();
-				p.StartInfo.UseShellExecute = false;
-				p.StartInfo.FileName = helper_path;
+				SafeProcess p = new SafeProcess ();
+				p.Arguments = new string [] { helper_path };
 				p.Start ();
 
 				Logger.Log.Debug ("IndexHelper PID is {0}", p.Id);
 
-				// Poll the helper's socket.  If the 
+				// Poll the helper's socket.  Wait up to a minute
+				// (500 ms * 120 times) for the helper to be ready
+				// to handle requests.
+				Stopwatch watch = new Stopwatch ();
+				watch.Start ();
 				int poll_count = 0;
 				bool found_helper;
 				do {
-					Thread.Sleep (200);
+					Thread.Sleep (500);
 					++poll_count;
 					found_helper = CheckHelper ();
-				} while (poll_count < 20 
+				} while (poll_count < 120 
 					 && ! found_helper
 					 && ! Shutdown.ShutdownRequested);
+				watch.Stop ();
 				
 				if (! found_helper)
-					throw new Exception ("Couldn't launch helper process");
+					throw new Exception (String.Format ("Couldn't launch helper process {0}", p.Id));
+
+				Logger.Log.Debug ("Found IndexHelper ({0}) in {1}", p.Id, watch);
 			}
 		}
 	}
