@@ -44,14 +44,15 @@ namespace Search.Tiles {
 		{
 			Group = TileGroup.Conversations;
 
-			subject.LabelProp = Title = hit.GetFirstProperty ("dc:title");
-			from.LabelProp = "<b>" + GetAddress (hit) + "</b>";
+			Subject.LabelProp = Title = hit.GetFirstProperty ("dc:title");
+			From.LabelProp = "<b>" + GetAddress (hit) + "</b>";
 			try {
 				Timestamp = Utils.ParseTimestamp (hit.GetFirstProperty ("fixme:date"));
-				date.LabelProp = Utils.NiceShortDate (Timestamp);
+				Date.LabelProp = Utils.NiceShortDate (Timestamp);
 			} catch {}
 
-			AddAction (new TileAction (Catalog.GetString ("Send in Mail"), SendInMail));
+			if (Hit.GetFirstProperty ("fixme:client") == "evolution")
+				AddAction (new TileAction (Catalog.GetString ("Send in Mail"), SendInMail));
 		}
 
 		protected override void LoadIcon (Gtk.Image image, int size)
@@ -81,17 +82,17 @@ namespace Search.Tiles {
 		{
 			DetailsPane details = new DetailsPane ();
 
-			details.AddLabelPair (Catalog.GetString ("Subject:"),
-					      SubjectLabel.Text,
-					      0, 1);
-			details.AddLabelPair (Catalog.GetString ("From:"),
-					      GetAddress (Hit),
-					      1, 1);
-			details.AddLabelPair (Catalog.GetString ("Date Received:"),
-					      DateLabel.Text,
-					      2, 1);
+			bool sent = (Hit.GetFirstProperty ("fixme:isSent") != null);
 
-			details.AddSnippet (3, 1);
+			details.AddLabelPair (Catalog.GetString ("Subject:"), SubjectLabel.Text);
+
+			string label = sent ? Catalog.GetString ("To:") : Catalog.GetString ("From:");
+			details.AddLabelPair (label, GetAddress (Hit));
+
+			label = sent ? Catalog.GetString ("Date Sent:") : Catalog.GetString ("Date Received:");
+			details.AddLabelPair (label, Utils.NiceLongDate (Timestamp));
+
+			details.AddSnippet ();
 
 			return details;
 		}
@@ -121,9 +122,6 @@ namespace Search.Tiles {
 
 		public void SendInMail ()
 		{
-			if (Hit.GetFirstProperty ("fixme:client") != "evolution")
-				return;
-			
 			SafeProcess p = new SafeProcess ();
 			p.Arguments = new string [] { "evolution", String.Format ("{0};forward=attached", Hit.Uri) };
 
